@@ -2,7 +2,7 @@ package Plack::Middleware::Statsd;
 
 # ABSTRACT: send statistics to statsd
 
-# RECOMMEND PREREQ:  Net::Statsd::Tiny v0.2.0
+# RECOMMEND PREREQ:  Net::Statsd::Tiny v0.3.0
 
 use v5.10;
 
@@ -12,12 +12,11 @@ use warnings;
 use parent qw/ Plack::Middleware /;
 
 use Plack::Util;
-use Plack::Util::Accessor qw/ client /;
-use POSIX ();
+use Plack::Util::Accessor qw/ client sample_rate /;
 use Time::HiRes;
 use Try::Tiny;
 
-our $VERSION = 'v0.3.0';
+our $VERSION = 'v0.3.1';
 
 sub call {
     my ( $self, $env ) = @_;
@@ -35,6 +34,10 @@ sub call {
 
             return unless $client;
 
+            my $rate = $self->sample_rate;
+
+            $rate = undef if ( defined $rate ) && ( $rate >= 1 );
+
             my $histogram = $client->can('timing') // $client->can('timing_ms');
             my $increment = $client->can('increment');
             my $set_count = $client->can('set_add');
@@ -44,7 +47,7 @@ sub call {
                 my ( $method, @args ) = @_;
                 try {
                     return unless defined $method;
-                    $client->$method(@args);
+                    $client->$method( grep { defined $_ } @args );
                 }
                 catch {
                     if ($logger) {
@@ -59,24 +62,28 @@ sub call {
             my $elapsed = Time::HiRes::tv_interval($start);
 
             $measure->(
-                $histogram, 'psgi.response.time', POSIX::ceil( $elapsed * 1000 )
+                $histogram, 'psgi.response.time', $elapsed * 1000, $rate
             );
 
             if ( defined $env->{CONTENT_LENGTH} ) {
                 $measure->(
                     $histogram, 'psgi.request.content-length',
-                    $env->{CONTENT_LENGTH}
+                    $env->{CONTENT_LENGTH}, $rate
                 );
             }
 
             if ( my $method = $env->{REQUEST_METHOD} ) {
-                $measure->( $increment, 'psgi.request.method.' . $method );
+                $measure->(
+                    $increment, 'psgi.request.method.' . $method, $rate
+                );
             }
 
             if ( my $type = $env->{CONTENT_TYPE} ) {
                 $type =~ s#/#.#g;
                 $type =~ s/;.*$//;
-                $measure->( $increment, 'psgi.request.content-type.' . $type );
+                $measure->(
+                    $increment, 'psgi.request.content-type.' . $type, $rate
+                );
 
             }
 
@@ -105,10 +112,13 @@ sub call {
             if ( my $type = $h->get('Content-Type') ) {
                 $type =~ s#/#.#g;
                 $type =~ s/;.*$//;
-                $measure->( $increment, 'psgi.response.content-type.' . $type );
+                $measure->(
+                    $increment, 'psgi.response.content-type.' . $type, $rate
+                );
             }
 
-            $measure->( $increment, 'psgi.response.status.' . $res->[0] );
+            $measure->( $increment, 'psgi.response.status.' . $res->[0],
+                $rate );
 
             if (
                   $env->{'psgix.harakiri.supported'}
@@ -116,7 +126,7 @@ sub call {
                 : $env->{'psgix.harakiri.commit'}
               )
             {
-                $measure->( $increment, 'psgix.harakiri' );
+                $measure->( $increment, 'psgix.harakiri' );    # rate == 1
             }
 
             $measure->( $client->can('flush') );
@@ -142,7 +152,7 @@ Plack::Middleware::Statsd - send statistics to statsd
 
 =head1 VERSION
 
-version v0.3.0
+version v0.3.1
 
 =head1 SYNOPSIS
 
@@ -152,7 +162,8 @@ version v0.3.0
   builder {
 
     enable "Statsd",
-      client      => Net::Statsd::Client->new( ... );
+      client        => Net::Statsd::Tiny->new( ... ),
+      sampling_rate => 1.0;
 
     ...
 
@@ -181,7 +192,7 @@ to a statsd server.
 
 =head2 client
 
-This is a statsd client, such as an L<Net::Statsd::Client> object.
+This is a statsd client, such as an instance of L<Net::Statsd::Tiny>.
 
 If one is omitted, then it will default to one defined in the
 environment hash at C<psgix.monitor.statsd>.
@@ -190,7 +201,7 @@ C<psgix.monitor.statsd> will be set to the current client if it is not
 set.
 
 The only restriction on the client is that it has the same API as
-L<Net::Statsd::Client> or similar modules, by supporting the following
+L<Net::Statsd::Tiny> or similar modules, by supporting the following
 methods:
 
 =over
@@ -209,6 +220,9 @@ C<set_add>
 
 =back
 
+This has been tested with L<Net::Statsd::Lite> and
+L<Net::Statsd::Client>.
+
 Other statsd client modules may be used via a wrapper class.
 
 =head1 METRICS
@@ -259,7 +273,11 @@ A counter for the HTTP status code is incremented.
 
 =item C<psgi.response.time>
 
-The response time, in ms (rounded up using C<ceil>).
+The response time, in ms.
+
+As of v0.3.1, this is no longer rounded up to an integer. If this
+causes problems with your statsd daemon, then you may need to use a
+subclassed version of your statsd client to work around this.
 
 =item C<psgi.response.x-sendfile>
 
@@ -276,8 +294,8 @@ This counter is incremented when the harakiri flag is set.
 
 =back
 
-If you want to rename these, then you will need to use a wrapper
-class for the L</client>.
+If you want to rename these, or modify sampling rates, then you will
+need to use a wrapper class for the L</client>.
 
 =head1 EXAMPLES
 
@@ -297,6 +315,15 @@ You can access the configured statsd client from L<Catalyst>:
     $c->next::method(@_);
   }
 
+=head1 KNOWN ISSUES
+
+=head2 Support for older Perl versions
+
+This module requires Perl v5.10 or newer.
+
+Pull requests to support older versions of Perl are welcome. See
+L</SOURCE>.
+
 =head1 SEE ALSO
 
 L<Net::Statsd::Client>
